/**
 * Plot
 *
 * This class creates a plot object which can be used to display functions
 * (Line, VL, Vert classes) or matrix representations (matrix class).
 *
 * Currently has pan and zoom functionality (by changing x and y bounds) 
 *
 * Panning functionality, useful for function display, is only partially 
 * supported.
 *
 * @author Sarah Greer
 * @see Matrix
 * @see Line
 * @see VL
 * @see Vert
 *
 */


public class Plot{
  int nx, ny;   // number of x and y pixels
  int cx, cy;   // center of x and y
  color c;      // selected colors
  float minx, miny, maxx, maxy;     // plot x and y bounds
  float[] x, y;         // coordinates mapped to pixels
  float[] xmap, ymap;   // coordinated mapped to real values
  float dx, dy;         // spacing for plotting units

  public Plot(int nx, int ny, float minx, float miny, float maxx, float maxy, color c){
    this.c = c;
    this.nx = nx;
    this.ny = ny;
    cx = nx/2;
    cy = ny/2;

    this.minx = minx;
    this.miny = miny;
    this.maxx = maxx;
    this.maxy = maxy;

    x = new float[nx];
    y = new float[ny];

    xmap = new float[nx];
    ymap = new float[ny];

    updatePlot();
  }

  // updates the display
  private void updatePlot(){
    dx = (maxx - minx) / nx;
    dy = (maxy - miny) / ny;

    for (int i = 0; i < nx; i++){
      x[i] = minx + dx*i;
      xmap[i] = i;
    }
    for (int i = 0; i < ny; i++){
      y[i] = miny + dy*i;
      ymap[i] = i;
    }
  }

  // sets x bounds
  public void setXBounds(float minx, float maxx){
    this.minx = minx;
    this.maxx = maxx;
    updatePlot();
  }

  // sets y bounds
  public void setYBounds(float miny, float maxy){
    this.miny = miny;
    this.maxy = maxy;
    updatePlot();
  }

  // changes background color for plot
  public void changeColor(color c){
    c = c;
  }

  // display the plot
  public void display(){
    rectMode(CENTER);
    fill(c);
    stroke(c);
    rect(cx,cy,nx,ny);

    // write the bounds
    stroke(#696969);
    fill(#696969);
    textAlign(LEFT, CENTER);
    text(substr(str(ymax),5), cx+10, ymap[0]+15);
    text(substr(str(ymin),5), cx+10, ymap[ny-1]-15);
    text(substr(str(xmin),6), xmap[0]+10, cy+20);
    text(substr(str(xmax),5), xmap[nx-1]-55, cy+20);

    // set graph lines (for panning -- partially implemented)
    int y0ind = cy, x0ind = cx;
    float y0err = 0.1, x0err = 0.1;
    for (int i = 0; i < ny; i++){
      if (y[i] >= 0){
        if (y[i] < y0err){ 
          y0err = y[i];
          y0ind = i; 
        }
      } 
      if (i%100 == 0){
        strokeWeight(1);
        line(0-2,ymap[i],nx+2,ymap[i]);
      } else if (i%50 == 0){
        strokeWeight(1);
        line(nx-2,ymap[i],nx+2,ymap[i]);
        line(0-2,ymap[i],0+2,ymap[i]);
      }
    }

    // set graph lines
    strokeWeight(5);
    line(0-2,ymap[y0ind],nx+2,ymap[y0ind]);
    for (int i = 0; i < nx; i++){
      if (x[i] >= 0){
        if (x[i] < x0err){  
          x0err = x[i];
          x0ind = i; 
        }
      } 
      if (i%100 == 0){
        strokeWeight(1);
        line(xmap[i],0-2,xmap[i],ny+2);
      } else if (i%50 == 0){
        strokeWeight(1);
        line(xmap[i],ny-2,xmap[i],ny+2);
        line(xmap[i],0-2,xmap[i],0+2);
      }
    }

    strokeWeight(5);
    line(xmap[x0ind],0-2,xmap[x0ind],ny+2);
    strokeWeight(1);
  }
}
