/**
 * Newton's method visualization 
 *
 * This is a program to show the Newton-Raphson method for root finding for
 * various functions.
 *
 * Made with Processing (https://processing.org/)
 *
 * To modify the functions displayed, you can change the function call and 
 * bounds of the plot at the bottom of this file, which is labeled by:
 * MODIFY HERE
 *
 * @author Sarah Greer
 * www.sygreer.com
 *
 */

// objects
Plot plot;
Function func;
Button r;
RadioButton rb;
Line slpLine;

// zoom levels
int selected_prev = 0;
int zoomlevel = 0;

// colors for increasing iterations
color[] colors = {#ff00ff,
  #ff55aa,
  #ffaa55,
  #ffff00,
  #aaff55,
  #55ffaa,
  #00ffff};

// store iterations
ArrayList<Line> lines;
ArrayList<VL> verts;

// values for bounds
float xmin, xmax, ymin, ymax;
float xmin_orig, xmax_orig, ymin_orig, ymax_orig;

// selected function
int selectedButton;
int selected = 0;

// check if iterations have started
Boolean hasClicked = false;

// arrays to map function values onto pixel display
int[] ximap;
int maxiter=100;
int doneiter;
Boolean outOfBounds = false;

// initialize display
void setup(){
  size(1500,800);
  background(255);

  // default boundaries
  xmin = -TWO_PI;
  xmax = TWO_PI;
  ymin = -2;
  ymax = 2;

  // create objects
  plot = new Plot(1000,800,xmin, ymin, xmax, ymax, #d6ebf2);
  func = new Function(plot);
  r = new Button(1250, 750, 100, 50, #ff0000);

  // functions
  // MODIFY HERE TO CHANGE THE NAME OF THE FUNCTION FOR DISPLAY
  String[] names = new String[]{"sin(x)","cos(x)+1/2x^3","x^2 - 25","1/3x","x^3-2x+2"};
  rb = new RadioButton(1200, 175, 15, #999999, 30, names);

  // reset button
  r.setText(20, "Reset", #ffffff);

  // selected value
  selectedButton = rb.selectedButton();

  // store iterations
  lines = new ArrayList<Line>();
  verts = new ArrayList<VL>();

  // lower frame rate
  frameRate(10);

  // set default bounds
  xmin_orig = xmin;
  xmax_orig = xmax;
  ymin_orig = ymin;
  ymax_orig = ymax;

  ximap = new int[maxiter];
  ximap[0] = 0;
}

// draw display
void draw(){
  background(#ffffff);
  // selected value
  selectedButton = rb.selectedButton();

  // update display if function is changed 
  if (selected_prev != selectedButton){
    res();
    updateGraph();
    func.generateFunction();
  }

  // store flag if the function changed
  selected_prev = selectedButton;

  // display values
  plot.display();
  func.display();
  func.derLine();
  func.coords();

  // show lines
  for (int i = lines.size()-1; i >= 0; i--) { 
    Line slpLine = lines.get(i);
    slpLine.display();
    VL vert = verts.get(i);
    vert.display();
    //print("(" +slpLine.newton() + ", " +slpLine.ynewton() + ") ");
  }

  // cover iterations from GUI area
  fill(#ffffff);
  noStroke();
  rect(1250,400,500,800);

  // show buttons
  rb.display();

  // reset button
  if(r.isClicked()){
    res();
  }

  // text/description display
  textSize(20);
  textAlign(CENTER,CENTER);
  text("Functions",1225,125);
  textSize(30);
  text("Newton-Raphson Method",1250,25);
  textAlign(LEFT,CENTER);
  textSize(15);
  text("Select a function below and click anywhere on the graph to set the starting guess for the algorithm.",1250, 75, 400, 800);
  textSize(20);
  
  // start iterations
  if (hasClicked){
    float startY=375;
    int maxloop = doneiter + 1;
    if (outOfBounds){
      maxloop = doneiter;
      text("Iteration out of bounds." ,1100,startY + doneiter*20+10);
    }

    // display iteration values
    textSize(20);
    float[] x = func.x;
    float[] xi = new float[maxiter];
    x = func.x;
    xi = new float[maxiter];
    for (int i=0; i<maxloop; i++){
      fill(colors[i%7]);
      xi[i] = x[ximap[i]];
      text(str(i),1110,startY + i*20);
      text(substr(str(xi[i]),10),1150,startY + i*20);
      text(substr(str(functionForDisplay(xi[i])),13),1290,startY + i*20);
    }
    fill(#000000);
    text("n",1110,startY-30 );
    text("x",1195,startY-30 );
    text("y",1345,startY-30 );
    textSize(15);
    text("n",1207,startY-22 );
    text("n",1357,startY-22 );
    line(1140,startY-40,1140,startY+maxloop*20);
    line(1275,startY-40,1275,startY+maxloop*20);
    line(1090,startY-10,1435,startY-10);
  }
  r.display();

}

// set maximum size for string
String substr(String string, int leng){
  if(string.length() > leng){
    string = string.substring(0,leng);
  }
  return string;
}

// alias for updating bounds

// update the graph
void updateGraph(){
  changeBounds();
  plot.setXBounds(xmin,xmax);
  plot.setYBounds(ymin,ymax);
  func.graphUpdate();
  func.generateFunction();
  for (int i = lines.size()-1; i >= 0; i--) { 
    Line slpLine = lines.get(i);
    slpLine.graphUpdate();
    slpLine.generateFunction();

    VL vert = verts.get(i);
    vert.graphUpdate();
    vert.generateFunction();
  }
}

// reset the graph
void res(){
    for (int i = lines.size()-1; i >= 0; i--) { 
      Line slpLine = lines.get(i);
      lines.remove(i);

      VL vert = verts.get(i);
      verts.remove(i);
    }
    xmin = xmin_orig;
    xmax = xmax_orig;
    ymin = ymin_orig;
    ymax = ymax_orig;
    updateGraph();
    hasClicked = false;
    outOfBounds = false;
}

// start algorithm
void addIter(int xval){
  ximap = new int[maxiter];

  // loop through iterations
  for (int i=0; i<maxiter; i++){
    // make sure it's in bounds
    if(xval > 1000 || xval < 0){
      outOfBounds = true;
      doneiter=i;
      break;
    }

    // add next iteration
    ximap[i] = xval;
    lines.add(new Line(plot, func, xval)); 
    verts.add(new VL(plot, func, xval)); 
    VL vert = verts.get(i);
    Line slpLine = lines.get(i);
    slpLine.setColor(colors[i%7]);
    vert.setColor(colors[i%7]);

    if (vert.getNewX() == xval){ 
      doneiter=i;
      break;
    }
    xval=vert.getNewX();
  }
}

// select x0 when click
void mouseClicked() {
  if(hasClicked == false && mouseX<1000){
      addIter(mouseX);
      hasClicked = true;
  }
}


/* MODIFY HERE */

// MODIFY THIS TO CHANGE THE FUNCTION
// function for display
float functionForDisplay(float x){
  if (selectedButton == 0){
    return sin(x);
  } else if (selectedButton == 1){
    return cos(x) + 1/2.0*x*x*x;
  } else if (selectedButton == 2){
    return x*x - 25;
  } else if (selectedButton == 3){
    return 1/3.0*x;
  } else if (selectedButton == 4){
    return x*x*x - 2*x + 2;
  } else if (selectedButton == 5){
    return x*x*x - 2*x + 2;
  }
  else{
    return 0;
  }
}

// MODIFY THIS TO CHANGE THE BOUNDS
// bounds for each function
void changeBounds(){
  if (selectedButton == 0){
    xmin = -TWO_PI;
    xmax = TWO_PI;
    ymin = -2;
    ymax = 2;
  } else if (selectedButton == 1){
    xmin = -3;
    xmax = 3;
    ymin = -6;
    ymax = 6;
  } else if (selectedButton == 2){
    xmin = -10;
    xmax = 10;
    ymin = -30;
    ymax = 30;
  } else if (selectedButton == 3){
    xmin = -6;
    xmax = 6;
    ymin = -2;
    ymax = 2;
  } else if (selectedButton == 4){
    xmin = -3;
    xmax = 3;
    ymin = -5;
    ymax = 5;
  } else if (selectedButton == 5){
    xmin = -6;
    xmax = 6;
    ymin = -2;
    ymax = 2;
  }
}

